package polyglot.ext.polyj.types;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;

import polyglot.ext.jl.types.*;

import polyglot.ext.polyj.ast.*;
import polyglot.ext.polyj.visit.*;

import java.util.*;

/**
 * Instances of ParamType represent formal parameters of a
 * PolyJClassType.  Two ParamType instances are equal iff
 * they belong to the same PolyJClassType and are in the same
 * position in its arguments list.
 */
public interface ParamType extends ClassType {

    /** New enum for the kind of class type params are. */
    public static final ClassType.Kind PARAM = new ClassType.Kind("param");

    /**
     * Returns the PJCT this parameter constrains, or null
     * if the parameter constrains something other than a PJCT.
     */
    PolyJClassType constrainedClass();

    /**
     * Returns the type this parameter constrains.
     */
    Type constrainedType();

    /**
     * Returns the 0-based index of this parameter in its
     * constraining type.  For example, if this is "U",
     * and it constrains Pair[T,U], index() would return 1.
     */
    int index();

    /**
     * Add a where constraint to this parameter.  An imperative update.
     */
    void addConstraint(ClassConstraint c);
}
