package polyglot.ext.polyj.types;

import polyglot.frontend.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.ext.polyj.ast.*;

import polyglot.ext.jl.types.*;

import java.io.*;
import java.util.*;

public class PolyJClassType_c extends ParsedClassType_c implements PolyJClassType
{

    public PolyJClassType_c(TypeSystem ts, LazyClassInitializer init, 
                                  Source fromSource)
    {
        super(ts, init, fromSource);
	params = new TypedList(new LinkedList(), ParamType.class, false);
	this.ts = (PolyJTypeSystem)ts;
    }

    /**
     * The parameters of type, stored in the delegate to
     * eliminate the need for redundant implementation
     * of them in every subclass of PolyJClassType.
     */
    protected List /*of ParamType*/ params;
    
    /**
     * The type system, cached
     * here and cast to a PJTS.
     */
    protected transient PolyJTypeSystem ts;

    public List params() {
        return params;
    }
    
    public ParamType paramNamed(String name) {
        for (Iterator p = params().iterator(); p.hasNext();) {
            ParamType par = (ParamType) p.next();
            if (par.name().equals(name)) {
                return par;
            }
        }
        return null;
    }
    
    public boolean isPolymorphic() {
        return params.size() != 0;
    }
    
    public Type instantiate(List args) throws SemanticException {
        return ts.instantiate(this,    TypedList.copyAndCheck(args, Type.class, true));
    }
    
    public void addParam(String paramName) {
        int index = params.size();
        params.add(ts.ParamType(position(), this, paramName, index));
    }

    public void addConstraint(ClassConstraint c) throws SemanticException {
        String paramName = c.name();
        ParamType param = paramNamed(paramName);
        if (param == null) {
            throw new SemanticException("Unknown parameter " + paramName,
                c.position());
        }
        paramNamed(paramName).addConstraint(c);
    }

    
    public WrappedClassType wrap(ClassType oldEnclosingType, ClassType newEnclosingType, PolyJTypeWrapper tw) {
        return new PolyJSubstType_c(this, oldEnclosingType, newEnclosingType, tw);
    }    
}
